--[[
Name: TalentShow-1.0
Revision: $Rev: 24090 $
Author(s): Takadi (nekoreeve@cox.net)
Website: http://www.wowace.com/wiki/TalentShow-1.0
Documentation: http://www.wowace.com/wiki/TalentShow-1.0
SVN: svn://svn.wowace.com/wowace/trunk/TalentShow-1.0
Description: Library for collecting talent data.
Dependencies: AceLibrary, AceEvent-2.0, (optional) AceDebug-2.0
]]

local MAJOR_VERSION = "TalentShow-1.0"
local MINOR_VERSION = "$Revision: 24090 $"

if not AceLibrary then
	error(MAJOR_VERSION .. " requires AceLibrary.")
end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
	return
end

if not AceLibrary:HasInstance("AceEvent-2.0") then
	error(MAJOR_VERSION .. " requires AceEvent-2.0.")
end

local TalentShow = {}
AceLibrary("AceEvent-2.0"):embed(TalentShow)
if AceLibrary:HasInstance("AceDebug-2.0") then
	AceLibrary("AceDebug-2.0"):embed(TalentShow)
else
	self.Debug = function(...) return end
end

		--------------------------------
		-- TalentShow private functions --
		--------------------------------

function TalentShow:CHARACTER_POINTS_CHANGED(arg1)
	if arg1 == -1 then
		self:Update()
	end
end

		----------------------------
		-- TalentShow API functions --
		----------------------------

function TalentShow:Update()
	for tab = 1, GetNumTalentTabs(), 1 do
		local name, icon, points, _ = GetTalentTabInfo(tab)
		self.talents.byIndex[tab] = {
			["name"] = name,
			["icon"] = icon,
			["points"] = points
		}
		for idx = 1, GetNumTalents(tab), 1 do
			local name, icon, _, _, rank, full, _, _ = GetTalentInfo(tab, idx)
			self.talents.byIndex[tab][idx] = {
				["name"] = name,
				["icon"] = icon,
				["rank"] = rank,
				["full"] = full,
				["tab"] = tab,
				["index"] = idx
			}
			self.talents.byName[name] = self.talents.byIndex[tab][idx]
		end
	end
end

function TalentShow:GetTalentRanks(arg1, arg2)
	if type(arg1) == "string" then
		if type(self.talents.byName[arg1]) ~= "table" then
			self:Debug("GetTalentRank() - No such talent '%s'.", arg1)
			return nil
		end
		return self.talents.byName[arg1].rank, self.talents.byName[arg1].full
	elseif type(arg1) ~= "number" or type(arg2) ~= "number" then
		self:Debug("GetTalentRank() - Invalid argument(s).")
		return nil
	end
	if type(self.talents.byIndex[arg1][arg2]) ~= "table" then
		self:Debug("GetTalentRank() - No talent in tab '%d'" ..
			" with index '%d'.", tab, index)
		return nil
	end
	return self.talents.byIndex[arg1][arg2].rank,
		self.talents.byIndex[arg1][arg2].full
end

function TalentShow:GetTalentIndex(name)
	if type(name) ~= "string" then
		self:Debug("GetTalentIndex() - Invalid argument.")
		return nil, nil
	end
	if type(self.talents.byName[name]) ~= "table" then
		self:Debug("GetTalentIndex() - No such talent '%s'.", name)
		return nil, nil
	end
	return self.talents.byName[name].tab, self.talents.byName[name].index
end

function TalentShow:GetTalentName(tab, index)
	if type(tab) ~= "number" or type(index) ~= "number" then
		self:Debug("GetTalentName() - Invalid argument(s).")
		return nil
	end
	if type(self.talents.byIndex[tab]) ~= "table" then
		self:Debug("GetTalentName() - No talent tab '%d'.", tab)
		return nil, nil
	end
	if type(self.talents.byIndex[tab][index]) ~= "table" then
		self:Debug("GetTalentName() - No talent in tab '%d'" ..
			" with index '%d'.", tab, index)
		return nil, nil
	end
	return self.talents.byIndex[tab][index].name
end

function TalentShow:GetTalentIcon(arg1, arg2)
	if type(arg1) == "string" then
		if type(self.talents.byName[arg1]) ~= "table" then
			self:Debug("GetTalentIcon() - No such talent '%s'.", arg1)
			return nil
		end
		return self.talents.byName[arg1].icon
	elseif type(arg1) ~= "number" or type(arg2) ~= "number" then
		self:Debug("GetTalentIcon() - Invalid argument(s).")
		return nil
	end
	if type(self.talents.byIndex[arg1]) ~= "table" then
		self:Debug("GetTalentIcon() - No talent tab '%d'.", arg1)
		return nil, nil
	end
	if type(self.talents.byIndex[arg1][arg2]) ~= "table" then
		self:Debug("GetTalentIcon() - No talent in tab '%d'" ..
			" with index '%d'.", tab, index)
		return nil
	end
	return self.talents.byIndex[arg1][arg2].icon
end

function TalentShow:GetCurrentBuild()
	local build = ""
	for _, tab in ipairs(self.talents.byIndex) do
		for _, idx in ipairs(tab) do
			build = build .. tostring(idx.rank)
		end
	end
	return build
end

		------------------------------
		-- Ace Library Miscellaeous --
		------------------------------

local function activate(self, oldLib, oldDeactivate)
	if oldLib then
		self.talents = {
			["byIndex"] = oldLib.talents.byIndex,
			["byName"] = oldLib.talents.byName
		}
	else
		self.talents = {
			["byIndex"] = {},
			["byName"] = {}
		}
	end

	if oldDeactivate then
		oldDeactivate(oldLib)
	end

	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
	self:Update()
end

local function deactivate(self)
	self:UnregisterEvent("CHARACTER_POINTS_CHANGED")
end

AceLibrary:Register(
	TalentShow,
	MAJOR_VERSION,
	MINOR_VERSION,
	activate,
	deactivate
)
TalentShow = nil
